/**
 * 协议帧
 */
(function(window){

    function serial_scheme(){
        var self = serial_scheme.prototype;
        for(var i in self.options.codes){
            self.options.hash_diy_code[self.options.codes[i].id] = self.options.codes[i];
        }
    }

    serial_scheme.prototype.options = {
        scheme_tail  : '22', // 协议帧尾部
        scheme_header: '22', // 协议帧头部
        err_code     : 'FF', // 错误指令(当指定的数据位存在此指令代表单片机无法处理或错误)

        codes: { // 指令集
            OPEN_REGISTER_1: {code: '0101', id: '0101', explain: '电机打开成功', is_echoes: true}, // 继电器1(电机)打开
            OPEN_REGISTER_2: {code: '0102', id: '0102', explain: '料仓1打开成功', is_echoes: true}, // 继电器2(料仓1)打开
            OPEN_REGISTER_3: {code: '0103', id: '0103', explain: '料仓2打开成功', is_echoes: true}, // 继电器3(料仓2)打开
            OPEN_REGISTER_4: {code: '0104', id: '0104', explain: '取米门打开成功', is_echoes: true}, // 继电器4(取米门)打开
            OPEN_REGISTER_5: {code: '0105', id: '0105', explain: '转换打开成功', is_echoes: true}, // 继电器5(转换开)打开
            OPEN_REGISTER_6: {code: '0106', id: '0106', explain: '品转打开成功', is_echoes: true}, // 继电器6(品转)打开
            OPEN_REGISTER_7: {code: '0107', id: '0107', explain: '漏斗门打开成功', is_echoes: true}, // 继电器7(漏斗门)打开

            CLOSE_REGISTER_1: {code: '0201', id: '0201', explain: '电机关闭成功', is_echoes: true}, // 继电器1(电机)关闭
            CLOSE_REGISTER_2: {code: '0202', id: '0202', explain: '料仓1关闭成功', is_echoes: true}, // 继电器2(料仓1)关闭
            CLOSE_REGISTER_3: {code: '0203', id: '0203', explain: '料仓2关闭成功', is_echoes: true}, // 继电器3(料仓2)关闭
            CLOSE_REGISTER_4: {code: '0204', id: '0204', explain: '取米门关闭成功', is_echoes: true}, // 继电器4(取米门锁)关闭
            CLOSE_REGISTER_5: {code: '0205', id: '0205', explain: '转换关闭成功', is_echoes: true}, // 继电器5(转换开)关闭
            CLOSE_REGISTER_6: {code: '0206', id: '0206', explain: '品转关闭成功', is_echoes: true}, // 继电器6(品转)关闭
            CLOSE_REGISTER_7: {code: '0207', id: '0207', explain: '漏斗门关闭成功', is_echoes: true}, // 继电器7(漏斗门)关闭

            SENSOR_DATA: {code: '0300', id: '0300', explain: '获取传感器数据', is_echoes: true}, // 获取传感器数据

            READ_WEIGHT: {code: '010300000002C40B', id: '010300', explain: '读取重量', is_echoes: true}, // 读取重量

            CHECKED_BALANCE_1: {code: '010600120001E80F', id: '010600120001', explain: '校秤第1步', is_echoes: false}, // 校秤第1步（载重为 0 时向校准操作寄存器写入）
            CHECKED_BALANCE_2: {code: '010600120002A80E', id: '010600120002', explain: '校秤第2步', is_echoes: false}, // 校秤第2步 （载重为已知重量时向校准操作寄存器写入）
            CHECKED_BALANCE_3: {code: '01100008000204********F978', id: '011000080002', explain: '校秤第3步', is_echoes: false}, // 校秤第3步 （向砝码重量值寄存器写入砝码的重量）
        },

        receive_data: {origin_code: '', code: ''},

        hash_diy_code: {},
    };

    /**
     * 获取当前协议的头部
     * @return string
     */
    serial_scheme.prototype.get_scheme_header = function(){ return serial_scheme.prototype.options.scheme_header; };

    /**
     * 获取当前协议的尾部
     * @return string
     */
    serial_scheme.prototype.get_scheme_tail = function(){ return serial_scheme.prototype.options.scheme_tail; };

    /**
     * 拼接协议 头、尾部
     * @param string code 指令码
     * @return string
     */
    serial_scheme.prototype.join_scheme = function(code){ return serial_scheme.prototype.get_scheme_header() + code + serial_scheme.prototype.get_scheme_tail(); };

    /**
     * 检查单片机传送指令是否正确
     * @param string code 指令
     * @return mixed
     */
    serial_scheme.prototype.check_data = function(code){
        // 指令长度必须在8个字符以上,并且头尾帧和设置头尾帧一致
        // 如果指令长度等于8个字符则校验指令数据位是否出错
        var self = serial_scheme.prototype;
        code     = String(code).replace(/\s+/g,'').toLocaleUpperCase();

        self.options.receive_data.code = '';
        self.options.receive_data.origin_code = '';

        // 对比指令长度及首尾帧
        if(!((code.length == 8 || code.length == 18) && code.slice(0, 2) == self.options.scheme_header && code.slice(-2) == self.options.scheme_tail))
        {
            return false;
        }

        // 当长度为8位时查看是否有数据错误位（重量传感器无需验证，只要返回一定是数据正常）
        if(code.length == 8 && (code.slice(4, 6) == self.options.err_code))
        {
            return false;
        }

        self.options.receive_data.origin_code = code;
        self.options.receive_data.code = code.slice(2).slice(0, -2);

        return true;
    };

    /**
     * 解析指令
     * @return mixed
     */
    serial_scheme.prototype.parse_code = function(){
        var self  = serial_scheme.prototype;
        var code  = self.options.receive_data.code;
        var flags = code.slice(0, 2);

        if((flags == '01' || flags == '02') && code.length == 4){ // 继电器

            return {type: 1, msg: self.options.hash_diy_code[code].explain, codes: self.options.hash_diy_code[code]};

        }else if(flags == '03' && code.length == 4){ // 返回传感器数据
            var data = self.parse_sensor_data();

            return {type: 2, msg: ('传感器数据：左仓 ' + (data.left_warehouse ? '有粮' : '无粮') + ' 右仓 ' + (data.right_warehouse ? '有粮' : '无粮') +
            ' 漏斗门 ' + (data.take_rice_gate ? '打开' : '关闭') + ' 取米门 ' + (data.meter_switch ? '打开' : '关闭')), data: {left_warehouse: data.left_warehouse,
            right_warehouse: data.right_warehouse, take_rice_gate: data.take_rice_gate, meter_switch: data.meter_switch}, codes: self.options.hash_diy_code['0300']}
        }else if(flags == '01' && code.length == 18){ // 读取重量

            return {type: 3, msg: self.read_weight(code), codes: self.options.hash_diy_code['010300']};

        }else{
            return {type: -1, msg: '未知指令', codes: ''};
        }
    }

    /**
     * 解析传感器数据
     * 需要查看的数据位(从数据低位算起)
     * 返回传感器数据，数据低1位左仓，第2位右仓探头状态，1表示有粮，0表示无粮，
     * 第3位表示漏斗门，1表示打开，0表示关闭，第4表示取米门开关
     * @param return object
     */
    serial_scheme.prototype.parse_sensor_data = function(){
        var data_bytes = '0X' + serial_scheme.prototype.options.receive_data.code.substr(2);

        return {
            left_warehouse: ((data_bytes ^ 0X00) & 0X01) == 1,
            right_warehouse: (((data_bytes ^ 0X00) & 0X02) >> 1) == 1,
            take_rice_gate: (((data_bytes ^ 0X00) & 0X04) >> 2) == 1,
            meter_switch: (((data_bytes ^ 0X00) & 0X08) >> 3) == 1
        };
    }

    /**
     * 读取重量
     * @param return string
     */
    serial_scheme.prototype.read_weight = function(){
        var code = serial_scheme.prototype.receive_data.code;

        var fisrt_bytes = code.slice(6, 10);
        var last_bytes  = code.slice(10, 14);

        return ((Number('0X' + fisrt_bytes)).toString(10) * 100 / 100) + ((Number('0X' + last_bytes)).toString(10) * 100 / 100);
    }

    window.serial_scheme = serial_scheme;
})(window);
